#include <plib.h>
#include "NU32v2.h"
#include "Nokia5110.h"

unsigned short int adcVal;

int main(void) {

  SYSTEMConfig(SYS_FREQ, SYS_CFG_ALL);

  // configure for multi-vectored mode
  INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);

  // enable multi-vector interrupts 
  INTEnableSystemMultiVectoredInt(); 

  // initialize serial port and interrupt
  initSerialNU32v2();
  initLEDs(); 	// set LED pins as outputs
  LED0 = 1;
  LED1 = 0;

  // init and clear the LCD
  LcdInitialize();
  LcdClear();

  // configure and enable the ADC
  CloseADC10();	// ensure the ADC is off before setting the configuration

  // define setup parameters for OpenADC10
  // Turn module on | output in integer | trigger mode auto | enable  autosample
  #define PARAM1  ADC_MODULE_ON | ADC_FORMAT_INTG | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON

  // define setup parameters for OpenADC10
  // ADC ref external    | disable offset test    | enable scan mode | perform 1 sample | use one buffer | use MUXA mode
  #define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF

  // define setup parameters for OpenADC10
  // 				  use ADC internal clock | set sample time
  #define PARAM3  ADC_CONV_CLK_INTERNAL_RC | ADC_SAMPLE_TIME_15

  // define setup parameters for OpenADC10
  // set AN0 -> pin B0
  #define PARAM4	ENABLE_AN0_ANA

  // define setup parameters for OpenADC10
  // assign channels you don't want to scan
  #define PARAM5	SKIP_SCAN_AN1 | SKIP_SCAN_AN2 | SKIP_SCAN_AN3 | SKIP_SCAN_AN4 | SKIP_SCAN_AN5 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 | SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14 | SKIP_SCAN_AN15

  // use ground as neg ref for A | use AN0 (B0) for input A
  // configure to sample AN0
  SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF); // configure to sample AN0
  OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using parameter define above

  EnableADC10(); // Enable the ADC

  while ( ! mAD1GetIntFlag() ) { } // wait for the first conversion to complete so there will be valid data in ADC result registers

  // configure Timer 2 using internal clock with given prescaler
  OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_256, 62500);

  // set up the timer interrupt with a priority of 2
  ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);

  // write the value of a variable
  gotoXY(0,0);
  LcdString("Reading the");
  gotoXY(0,1);
  LcdString("pot at 5Hz:");

  LED0 = 0;
  LED1 = 1;

  while(1) {
    // nothing, use the ISR to echo any bytes recieved
    // and clear the LCD if a 'c' is received
  }

  return 0;
} // end main

/* ISR for Timer 2*/
void __ISR(_TIMER_2_VECTOR, ipl2) Timer2Handler(void){
  char str[12]; // buffer for string written to Nokia 5110

  adcVal = ReadADC10(0); // read the adc (adcVal is unsigned short int)

  gotoXY(10,3);
  sprintf(str, "ADC = %4d", adcVal); // %4d provides 4 spaces for the integer
  LcdString(str);

  float adcV;
  adcV = (float) adcVal * 3.3/1023.0;
  gotoXY(10,4);
  sprintf(str, "V = %5.3f", adcV); // %5.3f provides 5 spaces for the float with 3 after the decimal place
  LcdString(str);

  // clear the interrupt flag
  mT2ClearIntFlag();
}

// UART3 interrupt handler, priority level 2
void __ISR(_UART_3_VECTOR, ipl2) IntUart3Handler(void) {
  // Is this an RX interrupt?
  if(INTGetFlag(INT_SOURCE_UART_RX(UART3))){
    char data = UARTGetDataByte(UART3);

    // Echo what we just received
    PutCharacter(UART3, data); 

    // do something with the data
    if(data =='c') {
      // do something
      LcdClear();
    }

    // Clear the RX interrupt Flag
    INTClearFlag(INT_SOURCE_UART_RX(UART3)); 
  }

  // We don't care about TX interrupt
  if(INTGetFlag(INT_SOURCE_UART_TX(UART3))) {
    INTClearFlag(INT_SOURCE_UART_TX(UART3));
  }
}
